const { cli, testCommand } = require('./cli');

const PIPE = '|';
const BASHPROFILE = '~/.bash_profile';

async function gitClone(repoPath, destDir) {
  try {
    await cli('git', 'clone', `https://github.com/${repoPath}.git`, destDir);
  } catch (err) {
    console.log(`${err.message}`);
    throw err;
  }
}

async function echo(msg, redirectionPath = null) {
  try {
    await cli('echo', msg, `> ${redirectionPath}`);
  } catch (err) {
    console.log(`${err.message}`);
    throw err;
  }
}

// $ rbenv install -l | egrep "\s[0-9]\.[0-9]\.[0-9]$" | tail -1
async function getLatestRubyVersion() {
  try {
    await cli(
      'rbenv install -l',
      PIPE,
      'grep',
      '-e',
      's[0-9].[0-9].[0-9]$',
      PIPE,
      'tail -1',
    );
  } catch (err) {
    console.log(`${err.message}`);
    throw err;
  }
}

async function installRuby() {
  try {
    // await cli('yum update');
    await cli(
      ...[  'yum',
            'install',
            '-y',
            'git-core',
            'zlib',
            'zlib-devel',
            'gcc-c++',
            'patch',
            'readline',
            'readline-devel',
            'libyaml-devel',
            'libffi-devel',
            'openssl-devel',
            'make',
            'bzip2',
            'autoconf',
            'automake',
            'libtool',
            'bison',
            'curl',
            'sqlite-devel',
            'bash'
          ]
    );
    //gpg --keyserver hkp://pool.sks-keyservers.net --recv-keys 409B6B1796C275462A1703113804BB82D39DC0E3 7D2BAF1CF37B13E2069D6956105BD0E739499BDB

    await cli(...['gpg', 
            '--keyserver', 
            'hkp://pool.sks-keyservers.net', 
            '--recv-keys', 
            '409B6B1796C275462A1703113804BB82D39DC0E3 7D2BAF1CF37B13E2069D6956105BD0E739499BDB'
      ])
    //curl -sSL https://get.rvm.io | bash -s stable --ruby
    await cli('curl', '-sSL', 'https://get.rvm.io').pipe(await cli(bash))// 'bash', '-s', 'stable')
    await cli('rvm', 'install', '2.5.3')
    /**

    // $ git clone https://github.com/rbenv/rbenv.git ~/.rbenv
    await gitClone('rbenv/rbenv', '/usr/.rbenv');

    // echo 'pathmunge /usr/.rbenv/bin' > /etc/profile.d/ree.sh
    await echo('pathmunge /usr/.rbenv/bin', '/etc/profile.d/ree.sh');

    // echo 'eval "$(rbenv init -)"' >> ~/.BASHPROFILE
    await echo('sudo eval "$(rbenv init -)"', '/etc/profile.d/ree.sh');
    await cli('chmod', '+x', '/etc/profile.d/ree.sh')

    // source ~/.bash_profile
    // await cli('source', BASHPROFILE);
    // await cli('exec $SHELL')

    // git clone https://github.com/rbenv/ruby-build.git ~/.rbenv/plugins/ruby-build
    await gitClone('rbenv/ruby-build', '/usr/.rbenv/plugins/ruby-build');
    // await cli('exec $SHELL')
    // await cli('source', BASHPROFILE);
    */
  } catch (err) {
    console.log(`${err.message}`);
    throw err;
  }
}

module.exports = async (version = null) => {
  // let versionNumber = '2.6.1';
  try {
    await installRuby();
    console.log("Successfully installed ruby")
    // await testCommand('ruby')

    // versionNumber = version || (await getLatestRubyVersion());

    /**
    await cli('rbenv', 'install', versionNumber);
    await cli('rbenv', 'global', versionNumber);

    // Env for apps
    await echo('export RACK_ENV=production', BASHPROFILE); // Generic for all Rack apps
    await echo('export RAILS_ENV="$RACK_ENV"', BASHPROFILE); // ROR
    await echo('export APP_ENV="$RACK_ENV"', BASHPROFILE); // Sinatra
    await echo('export RAILS_LOG_TO_STDOUT=true', BASHPROFILE); // ROR
    await cli('ruby', '-v')
    console.log("RUBY")
    // console.log(versionNumber)
    */
  } catch (err) {
    throw err
  }
  // TODO: Add Support for JIT compilation

  return cli('ruby', '-v')
};
